@@ -77,7 +77,7 @@ namespace Lum {
         Color negativeButtonBackgroundColor;
 
       public:
-        Theme(::Lum::OS::Display *display);
+        Theme(OS::Display *display);
         ~Theme();
 
         bool HasFontSettings() const;
@@ -88,13 +88,30 @@ namespace Lum {
         std::wstring GetFixedFontName() const;
         size_t GetFixedFontSize() const;
 
-        ScrollKnobMode GetScrollKnobMode() const;
-        ToolbarPosition GetToolbarPosition() const;
+        void GetObjectSize(ObjectType object,
+                           size_t& width,
+                           size_t& height) const;
+
+        bool NeedsObjectParentDraw(ObjectType object) const;
+
+        void DrawObject(OS::DrawInfo* draw,
+                        ObjectType object,
+                        int oX, int oY, size_t oWidth, size_t oHeight,
+                        int x, int y, size_t width, size_t height);
+
+        size_t GetSpaceHorizontal(OS::Display::Space space) const;
+        size_t GetSpaceVertical(OS::Display::Space space) const;
         size_t GetMinimalButtonWidth() const;
         size_t GetFirstTabOffset() const;
         size_t GetLastTabOffset() const;
         size_t GetEntryHorizTextOffset() const;
         size_t GetEntryVertTextOffset() const;
+        size_t GetDragStartSensitivity() const;
+        size_t GetMouseClickHoldSensitivity() const;
+
+        ScrollKnobMode GetScrollKnobMode() const;
+        ToolbarPosition GetToolbarPosition() const;
+        bool ShowButtonImages() const;
       };
     }
   }
