@@ -28,6 +28,8 @@
   #include <direct.h>
   #include <shlobj.h>
   #include <Lum/OS/Win32/OSAPI.h>
+#else
+  #include <dirent.h>
 #endif
 
 #if defined(HAVE_UNISTD_H)
@@ -756,5 +758,198 @@ namespace Lum {
       // TODO: Under Windows we have to compare case-insensitive
       return a.GetPath()!=b.GetPath();
     }
+
+    DirScanner::DirScanner(const Path& path)
+      : path(path)
+    {
+      // no code
+    }
+
+    DirScanner::~DirScanner()
+    {
+    }
+
+#if defined(HAVE_FINDFIRSTFILEW)
+    class Win32DirScanner : public DirScanner
+    {
+    private:
+      bool   initialized;
+      HANDLE handle;
+      Status status;
+
+    public:
+      Win32DirScanner(const Path& path)
+       : DirScanner(path),
+         initialized(false),
+         handle(INVALID_HANDLE_VALUE)
+      {
+        // no code
+      }
+
+      bool GetNext(Entry& entry, Status& status)
+      {
+        WIN32_FIND_DATAW data;
+
+        if (initialized && handle==INVALID_HANDLE_VALUE) {
+          status=this->status;
+          return false;
+        }
+        else if (!initialized) {
+          std::wstring tmp(path.GetPath());
+
+          initialized=true;
+
+          if (tmp.length()>0 && tmp[tmp.length()-1]!=L'\\'&& tmp[tmp.length()-1]!=L'/') {
+            tmp.append(1,L'\\');
+          }
+          tmp.append(1,L'*');
+
+          handle=FindFirstFileW(tmp.c_str(),&data);
+          if (handle==INVALID_HANDLE_VALUE) {
+            this->status.SetToCurrentW32Error();
+            return false;
+          }
+        }
+        else {
+          if (!FindNextFileW(handle,&data)) {
+            if (GetLastError()!=ERROR_NO_MORE_FILES) {
+              this->status.SetToCurrentW32Error();
+            }
+
+            FindClose(handle);
+
+            return false;
+          }
+        }
+
+        entry.name=data.cFileName;
+        entry.isDir=data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY;
+        entry.isLink=false;
+        entry.isHidden=data.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN;
+
+        entry.size=data.nFileSizeLow;
+        entry.timeModification.SetTime(data.ftLastWriteTime);
+
+        entry.user=0;
+        entry.user|=rightRead;
+        if (!(data.dwFileAttributes & FILE_ATTRIBUTE_READONLY)) {
+          entry.user|=rightWrite;
+        }
+
+        entry.group=0;
+        entry.other=0;
+
+        return true;
+      }
+    };
+
+#else
+    class UnixDirScanner : public DirScanner
+    {
+    private:
+      bool   initialized;
+      DIR    *dir;
+      Status status;
+
+    public:
+      UnixDirScanner(const Path& path)
+       : DirScanner(path),
+         initialized(false),
+         dir(NULL)
+      {
+        // no code
+      }
+
+      bool GetNext(Entry& entry, Status& status)
+      {
+        if (initialized && dir==NULL) {
+          status=this->status;
+          return false;
+        }
+        else if (!initialized) {
+          initialized=true;
+          dir=opendir(Lum::Base::WStringToString(path.GetPath()).c_str());
+
+          if (dir==NULL) {
+            this->status.SetToCurrentErrno();
+            return false;
+          }
+        }
+
+        struct dirent *entr;
+        struct stat   info;
+
+        entr=readdir(dir);
+
+        if (entr==NULL) {
+          // TODO: Error case?
+          closedir(dir);
+          return false;
+        }
+
+        if (stat(entr->d_name,&info)!=0) {
+          closedir(dir);
+          return false;
+        }
+
+        entry.name=Base::StringToWString(entr->d_name);
+        entry.isDir=S_ISDIR(info.st_mode);
+        if (lstat(entr->d_name,&info)==0) {
+          entry.isLink=S_ISLNK(info.st_mode);
+        }
+        else {
+          entry.isLink=false;
+        }
+        entry.isHidden=entry.name.length()>0 && entry.name[0]==L'.';
+
+        entry.size=info.st_size;
+        entry.timeModification.SetTime(info.st_mtime);
+
+        entry.user=0;
+        if (info.st_mode & S_IRUSR) {
+          entry.user|=rightRead;
+        }
+        if (info.st_mode & S_IWUSR) {
+          entry.user|=rightWrite;
+        }
+        if (info.st_mode & S_IXUSR) {
+          entry.user|=rightExecute;
+        }
+
+        entry.group=0;
+        if (info.st_mode & S_IRGRP) {
+          entry.group|=rightRead;
+        }
+        if (info.st_mode & S_IWGRP) {
+          entry.group|=rightWrite;
+        }
+        if (info.st_mode & S_IXGRP) {
+          entry.group|=rightExecute;
+        }
+
+        entry.other=0;
+        if (info.st_mode & S_IROTH) {
+          entry.other|=rightRead;
+        }
+        if (info.st_mode & S_IWOTH) {
+          entry.other|=rightWrite;
+        }
+        if (info.st_mode & S_IXOTH) {
+          entry.other|=rightExecute;
+        }
+
+        return true;
+      }
+    };
+#endif
+
+    DirScanner* DirScanner::Create(const Path& path)
+    {
+#if defined(HAVE_FINDFIRSTFILEW)
+      return new Win32DirScanner(path);
+#else
+      return new UnixDirScanner(path);
+#endif
+    }
   }
 }
