@@ -95,6 +95,7 @@ namespace Lum {
     value(value)
   {
     SetCanFocus(true);
+    RequestFocus();
     SetCanDrawMouseActive(true);
 
     value->SetParent(this);
@@ -171,8 +172,7 @@ namespace Lum {
   void Combo::CalcSize()
   {
     /* Delegate Focus visualisation to value class, if possible */
-    if (value->CanFocus()) {
-      value->RequestFocus();
+    if (value->RequestedFocus()) {
       UnrequestFocus();
     }
 
@@ -261,7 +261,6 @@ namespace Lum {
     combo->table->SetMinHeight(Base::Size::stdCharHeight,1);
     combo->table->SetMaxWidth(Base::Size::screenVRel,40);
     combo->table->SetMaxHeight(Base::Size::screenVRel,40);
-    combo->table->RequestFocus();
     combo->table->SetModel(combo->tableModel);
     combo->table->SetSelection(combo->selection);
     if (combo->tableModel.Valid() && combo->tableModel->GetColumns()==1) {
