@@ -17,6 +17,7 @@
 #include <Lum/ButtonRow.h>
 #include <Lum/Combo.h>
 #include <Lum/DirSelect.h>
+#include <Lum/Menu.h>
 #include <Lum/Panel.h>
 #include <Lum/PopupGroup.h>
 #include <Lum/Space.h>
@@ -89,7 +90,6 @@ namespace Lum {
         table->SetMinHeight(Base::Size::stdCharHeight,1);
         table->SetMaxWidth(Base::Size::screenVRel,40);
         table->SetMaxHeight(Base::Size::screenVRel,40);
-        table->RequestFocus();
         table->SetHeaderModel(headerModel);
         table->SetSelection(selection);
         table->SetModel(roots);
@@ -184,6 +184,7 @@ namespace Lum {
       path(new Model::Path()),
       full(new Model::String()),
       filter(new Model::SizeT()),
+      showHidden(new Model::Boolean()),
       okAction(new Model::Action()),
       reloadAction(new Model::Action()),
       parentAction(new Model::Action()),
@@ -198,16 +199,24 @@ namespace Lum {
 
       tmp.SetNativeDir(Base::Path::GetCWD());
 
+      // We switch notification of, since we do not want to trigger
+      // a call to Load() yet...
       path->Off();
 
       path->Set(tmp);
       full->Set(Base::Path::GetCWD());
 
+      // We switch notification of, since we do not want to trigger
+      // a call to Load() yet...
+      showHidden->Off();
+      showHidden->Set(false);
+
       AttachModel(GetOpenedAction());
       AttachModel(GetClosedAction());
       AttachModel(path);
       AttachModel(full);
       AttachModel(filter);
+      AttachModel(showHidden);
       AttachModel(okAction);
       AttachModel(reloadAction);
       AttachModel(parentAction);
@@ -226,11 +235,12 @@ namespace Lum {
       UnattachModel(parentAction);
       UnattachModel(reloadAction);
       UnattachModel(okAction);
+      UnattachModel(showHidden);
       UnattachModel(filter);
       UnattachModel(full);
       UnattachModel(path);
       UnattachModel(GetClosedAction());
-      AttachModel(GetOpenedAction());
+      UnattachModel(GetOpenedAction());
     }
 
     void File::SetOptions(Options* options)
@@ -255,6 +265,7 @@ namespace Lum {
       Button           *button;
       ButtonRow        *row;
       DirSelect        *dirSel;
+      Menu             *popup;
       Panel            *hPanel,*vPanel;
       Space            *space;
       WindowGroup      *wGroup;
@@ -278,7 +289,6 @@ namespace Lum {
       table=new Table();
       table->SetFlex(true,true);
       table->SetMinWidth(Base::Size::stdCharWidth,40);
-      table->RequestFocus();
       table->SetShowHeader(true);
       table->GetTableView()->SetAutoFitColumns(true);
       table->GetTableView()->SetAutoHSize(true);
@@ -289,6 +299,11 @@ namespace Lum {
       table->SetModel(dirList);
       table->SetHeaderModel(header);
 
+      popup=new Menu();
+      popup->SetParent(GetWindow());
+      popup->AddBoolItem(_l(L"DLG_FILE_SHOW_HIDDEN",L"Show _hidden files"),showHidden,OS::qualifierControl,L"h");
+      table->SetMenu(popup->GetWindow());
+
       vPanel->Add(table);
 
       vPanel->Add(new VSpace());
@@ -296,9 +311,8 @@ namespace Lum {
       hPanel=new HPanel();
       hPanel->SetFlex(true,false);
 
-      locations=new Lum::Button();
+      locations=new Button();
       locations->SetFlex(false,true);
-      locations->RequestFocus();
       locations->SetModel(locationsAction);
       locations->SetText(_l(L"DLG_FILE_LOCATIONS",L"_Locations"));
       hPanel->Add(locations);
@@ -318,7 +332,6 @@ namespace Lum {
 
         combo=new IndexCombo();
         combo->SetWidth(Base::Size::stdCharWidth,15);
-        combo->RequestFocus();
         combo->SetModel(filter);
         combo->SetTableModel(table);
         hPanel->Add(combo);
@@ -332,14 +345,12 @@ namespace Lum {
 
       fullStr=new String();
       fullStr->SetFlex(true,false);
-      fullStr->RequestFocus();
       fullStr->SetModel(full);
       fullStr->SetReturnAction(fullAction);
       hPanel->Add(fullStr);
 
       button=new Button();
       button->SetFlex(false,true);
-      button->RequestFocus();
       button->SetModel(reloadAction);
       button->SetText(_l(L"DLG_FILE_RELOAD",L"_Reload"));
       hPanel->Add(button);
@@ -352,7 +363,6 @@ namespace Lum {
 
       button=new Button;
       button->SetFlex(true,true);
-      button->RequestFocus();
       button->SetModel(okAction);
       button->SetText(_ld(dlgButtonOk));
       button->SetType(Button::typeCommit);
@@ -360,7 +370,6 @@ namespace Lum {
 
       button=new Button;
       button->SetFlex(true,true);
-      button->RequestFocus();
       button->SetModel(GetClosedAction());
       button->SetText(_ld(dlgButtonCancel));
       button->SetType(Button::typeCancel);
@@ -375,7 +384,6 @@ namespace Lum {
         wGroup->SetMinWidth(Base::Size::screenVRel,80);
       }
 
-
       wGroup->SetMain(vPanel);
 
       SetTop(wGroup);
@@ -417,7 +425,7 @@ namespace Lum {
       dirList->Off();
       table->GetTableView()->GetSelection()->Clear();
       dirList->SetDirectory(path->Get().GetDir(),
-                            false,
+                            !showHidden->Get(),
                             options->mode==Options::modeDirectory);
 
       table->GetTableView()->GetVAdjustment()->SetTop(1);
@@ -450,7 +458,7 @@ namespace Lum {
       path->Set(newPath);
     }
 
-    bool File::GetSelection(Model::Dir::Entry& entry) const
+    bool File::GetSelection(Base::DirScanner::Entry& entry) const
     {
       size_t line;
 
@@ -470,7 +478,7 @@ namespace Lum {
     */
     void File::OnSelectionChange()
     {
-      Model::Dir::Entry entry;
+      Base::DirScanner::Entry entry;
 
       if (!GetSelection(entry)) {
         return;
@@ -479,13 +487,13 @@ namespace Lum {
       Base::Path newPath(path->Get());
 
       if (options->mode==Options::modeDirectory) {
-        if (entry.type==Model::Dir::dirType) {
+        if (entry.isDir) {
           newPath.AppendDir(entry.name);
           full->Set(newPath.GetDir(true));
         }
       }
       else {
-        if (entry.type==Model::Dir::dirType) {
+        if (entry.isDir) {
           full->Set(newPath.GetDir(true));
         }
         else {
@@ -551,7 +559,7 @@ namespace Lum {
     */
     void File::OnOk()
     {
-      Model::Dir::Entry entry;
+      Base::DirScanner::Entry entry;
 
       if (!GetSelection(entry)) {
         return;
@@ -560,7 +568,7 @@ namespace Lum {
       Base::Path newPath(full->Get());
 
       if (options->mode==Options::modeDirectory) {
-        if (entry.type==Model::Dir::dirType) {
+        if (entry.isDir) {
           if (newPath.Exists() && newPath.IsDir()) {
             result=true;
             Exit();
@@ -568,7 +576,7 @@ namespace Lum {
         }
       }
       else {
-        if (entry.type==Model::Dir::dirType) {
+        if (entry.isDir) {
           newPath.AppendDir(entry.name);
           path->Set(newPath);
         }
@@ -584,10 +592,10 @@ namespace Lum {
 
     void File::OnDoubleClick()
     {
-      Model::Dir::Entry entry;
+      Base::DirScanner::Entry entry;
 
       if (GetSelection(entry)) {
-        if (entry.type==Model::Dir::dirType) {
+        if (entry.isDir) {
           GotoChildDirectory(entry.name);
         }
         else {
@@ -602,6 +610,9 @@ namespace Lum {
         full->Set(path->Get().GetPath());
         Load();
       }
+      if (model==showHidden) {
+        Load();
+      }
       else if (model==okAction  && okAction->IsFinished()) {
         OnOk();
       }
@@ -611,6 +622,7 @@ namespace Lum {
       }
       else if (model==GetOpenedAction() && GetOpenedAction()->IsFinished()) {
         path->On();
+        showHidden->On(false);
         SetFocus(table->GetTableView());
       }
       else if (model==reloadAction && reloadAction->IsFinished()) {
