@@ -19,27 +19,14 @@
 
 #include <Lum/LED.h>
 
-#include <Lum/OS/Display.h>
+#include <Lum/OS/Theme.h>
 
 namespace Lum {
 
-  static LED::Prefs *prefs=new ::Lum::LED::Prefs();
-
-  LED::Prefs::Prefs()
-  : ImageState::Prefs(L"LED")
-  {
-    // no code
-  }
-
-  void LED::Prefs::Initialize()
-  {
-    ImageState::Prefs::Initialize();
-
-    image=OS::display->GetImage(OS::Display::ledImageIndex);
-  }
+  static ImageState::Prefs *prefs=new ::Lum::ImageState::Prefs(L"LED");
 
   LED::LED()
-  : ImageState(false)
+   : ImageState(OS::Theme::objLED,false)
   {
     SetPrefs(::Lum::prefs);
   }
