@@ -525,7 +525,7 @@ namespace Lum {
         brush=::CreateSolidBrush(backgroundColor);
 
         wClass.hCursor=::LoadCursorA(0,IDC_ARROW);
-        wClass.hIcon=::LoadIconA(instance,(CHAR*)1/*iconId*/);
+        wClass.hIcon=::LoadIconA(instance,MAKEINTRESOURCE(101));
         wClass.lpszMenuName=NULL;
         wClass.lpszClassName="IlluminationWindowClass";
         wClass.hbrBackground=0;
