@@ -28,17 +28,17 @@
 #include <Lum/OS/Win32/Display.h>
 #include <Lum/OS/Win32/DrawInfo.h>
 
-#define MENU_BARBACKGROUND   7
-#define MENU_BARITEM         8
-#define MENU_POPUPCHECK     11
-#define MENU_UNKNOWN1       12
-#define MENU_POPUPITEM      14
-#define MENU_SEPARATOR      15
-#define MENU_POPUPSUBMENU   16
-#define MENU_SYSTEMCLOSE    17
-#define MENU_SYSTEMMAXIMISE 18
-#define MENU_SYSTEMMINIMIZE 19
-#define MENU_SYSTEMRESTOREE 20
+#define MENU_BARBACKGROUND         7
+#define MENU_BARITEM               8
+#define MENU_POPUPCHECK           11
+#define MENU_POPUPCHECKBACKGROUND 12
+#define MENU_POPUPITEM            14
+#define MENU_SEPARATOR            15
+#define MENU_POPUPSUBMENU         16
+#define MENU_SYSTEMCLOSE          17
+#define MENU_SYSTEMMAXIMISE       18
+#define MENU_SYSTEMMINIMIZE       19
+#define MENU_SYSTEMRESTOREE       20
 
 namespace Lum {
   namespace OS {
@@ -220,14 +220,6 @@ namespace Lum {
         : image(image)
       {
         switch (image) {
-        case Display::checkImageIndex:
-          width=GetSystemMetrics(SM_CXMENUCHECK);
-          height=GetSystemMetrics(SM_CYMENUCHECK);
-          break;
-        case Display::radioImageIndex:
-          width=GetSystemMetrics(SM_CXMENUCHECK);
-          height=GetSystemMetrics(SM_CYMENUCHECK);
-          break;
         case Display::hknobImageIndex:
           width=GetSystemMetrics(SM_CXHTHUMB);
           height=GetSystemMetrics(SM_CYHSCROLL);
@@ -323,6 +315,34 @@ namespace Lum {
             height=GetSystemMetrics(SM_CYMENUCHECK);
           }
           break;
+        case Display::menuCheckImageIndex:
+          if (HasTheming() && menuTheme!=NULL) {
+            SIZE size;
+
+            if (MyGetThemePartSize(menuTheme,dynamic_cast<Display*>(display)->hdc,MENU_POPUPCHECK,1,NULL,TS_TRUE,&size)==S_OK) {
+              width=size.cx;
+              height=size.cy;
+            }
+          }
+          else {
+            width=GetSystemMetrics(SM_CXMENUCHECK);
+            height=GetSystemMetrics(SM_CYMENUCHECK);
+          }
+          break;
+        case Display::menuRadioImageIndex:
+          if (HasTheming() && menuTheme!=NULL) {
+            SIZE size;
+
+            if (MyGetThemePartSize(menuTheme,dynamic_cast<Display*>(display)->hdc,MENU_POPUPCHECK,1,NULL,TS_TRUE,&size)==S_OK) {
+              width=size.cx;
+              height=size.cy;
+            }
+          }
+          else {
+            width=GetSystemMetrics(SM_CXMENUCHECK);
+            height=GetSystemMetrics(SM_CYMENUCHECK);
+          }
+          break;
         case Display::closeImageIndex:
           if (HasTheming() && menuTheme!=NULL) {
             SIZE size;
@@ -337,10 +357,6 @@ namespace Lum {
             height=GetSystemMetrics(SM_CYMENUCHECK)-4;
           }
           break;
-        case Display::ledImageIndex:
-          width=GetSystemMetrics(SM_CXMENUCHECK);
-          height=GetSystemMetrics(SM_CYMENUCHECK);
-          break;
         }
       }
 
@@ -378,68 +394,6 @@ namespace Lum {
         SetTextAlign(dc,TA_TOP+TA_LEFT);
 
         switch (image) {
-        case Display::checkImageIndex:
-          if (HasTheming()) {
-            if (draw->selected) {
-              if (draw->activated) {
-                style=CBS_CHECKEDHOT;
-              }
-              else if (draw->disabled) {
-                style=CBS_CHECKEDDISABLED;
-              }
-              else {
-                style=CBS_CHECKEDNORMAL;
-              }
-            }
-            else if (draw->activated) {
-              style=CBS_UNCHECKEDHOT;
-            }
-              else if (draw->disabled) {
-                style=CBS_UNCHECKEDDISABLED;
-              }
-            else {
-              style=CBS_UNCHECKEDNORMAL;
-            }
-
-            /* ignore */ MyDrawThemeBackground(buttonTheme,dc,BP_CHECKBOX,style,&rect,NULL);
-          }
-          else {
-           /* ignore */ ::DrawFrameControl(dc,&rect,
-                                            DFC_BUTTON,
-                                            DFCS_BUTTONCHECK | (draw->selected ? DFCS_CHECKED : 0));
-          }
-          break;
-        case Display::radioImageIndex:
-          if (HasTheming()) {
-            if (draw->selected) {
-              if (draw->activated) {
-                style=RBS_CHECKEDHOT;
-              }
-              else if (draw->disabled) {
-                style=RBS_CHECKEDDISABLED;
-              }
-              else {
-                style=RBS_CHECKEDNORMAL;
-              }
-            }
-            else if (draw->activated) {
-              style=RBS_UNCHECKEDHOT;
-            }
-            else if (draw->disabled) {
-              style=RBS_UNCHECKEDDISABLED;
-            }
-            else {
-              style=RBS_UNCHECKEDNORMAL;
-            }
-
-            /* ignore */ MyDrawThemeBackground(buttonTheme,dc,BP_RADIOBUTTON,style,&rect,NULL);
-          }
-          else {
-            /* ignore */ ::DrawFrameControl(dc,&rect,
-                                            DFC_BUTTON,
-                                            DFCS_BUTTONRADIO | (draw->selected ? DFCS_CHECKED : 0));
-          }
-          break;
         case Display::hknobImageIndex:
           if (HasTheming()) {
             if (draw->selected) {
@@ -829,6 +783,44 @@ namespace Lum {
             draw->PopForeground();
           }
           break;
+        case Display::menuCheckImageIndex:
+          if (HasTheming() && menuTheme!=NULL) {
+            if (draw->selected) {
+              if (draw->disabled) {
+                /* ignore */ MyDrawThemeBackground(menuTheme,dc,MENU_POPUPCHECKBACKGROUND,1,&rect,NULL);
+                /* ignore */ MyDrawThemeBackground(menuTheme,dc,MENU_POPUPCHECK,2,&rect,NULL);
+              }
+              else {
+                /* ignore */ MyDrawThemeBackground(menuTheme,dc,MENU_POPUPCHECKBACKGROUND,2,&rect,NULL);
+                /* ignore */ MyDrawThemeBackground(menuTheme,dc,MENU_POPUPCHECK,1,&rect,NULL);
+              }
+            }
+          }
+          else {
+           /* ignore */ ::DrawFrameControl(dc,&rect,
+                                            DFC_BUTTON,
+                                            DFCS_BUTTONCHECK | (draw->selected ? DFCS_CHECKED : 0));
+          }
+          break;
+        case Display::menuRadioImageIndex:
+          if (HasTheming() && menuTheme!=NULL) {
+            if (draw->selected) {
+              if (draw->disabled) {
+                /* ignore */ MyDrawThemeBackground(menuTheme,dc,MENU_POPUPCHECKBACKGROUND,1,&rect,NULL);
+                /* ignore */ MyDrawThemeBackground(menuTheme,dc,MENU_POPUPCHECK,4,&rect,NULL);
+              }
+              else {
+                /* ignore */ MyDrawThemeBackground(menuTheme,dc,MENU_POPUPCHECKBACKGROUND,2,&rect,NULL);
+                /* ignore */ MyDrawThemeBackground(menuTheme,dc,MENU_POPUPCHECK,3,&rect,NULL);
+              }
+            }
+          }
+          else {
+            /* ignore */ ::DrawFrameControl(dc,&rect,
+                                            DFC_BUTTON,
+                                            DFCS_BUTTONRADIO | (draw->selected ? DFCS_CHECKED : 0));
+          }
+          break;
         case Display::closeImageIndex:
           if (HasTheming() && menuTheme!=NULL) {
             /* ignore */ MyDrawThemeBackground(menuTheme,dc,MENU_SYSTEMCLOSE,1,&rect,NULL);
@@ -846,27 +838,6 @@ namespace Lum {
             draw->PopPen();
           }
           break;
-        case Display::ledImageIndex:
-          if (draw->selected) {
-            draw->PushForeground(GetSysColor(COLOR_HIGHLIGHT));
-            draw->FillArc(x,y,w-1,h-1,0*64,360*64);
-            draw->PopForeground();
-          }
-          else {
-            draw->PushForeground(GetSysColor(COLOR_3DSHADOW));
-            draw->FillArc(x,y,w-1,h-1,0*64,360*64);
-            draw->PopForeground();
-          }
-
-          draw->PushForeground(GetSysColor(COLOR_3DHIGHLIGHT));
-          draw->DrawArc(x+w / 4,y+h / 4,
-                       w / 2,h / 2,60*64,90*64);
-          draw->PopForeground();
-
-          draw->PushForeground(GetSysColor(COLOR_3DSHADOW));
-          draw->DrawArc(x,y,w-1,h-1,0*64,360*64);
-          draw->PopForeground();
-          break;
         }
 
         SetTextAlign(dc,TA_BASELINE+TA_LEFT);
@@ -946,6 +917,43 @@ namespace Lum {
         case Display::positiveButtonBackgroundFillIndex:
         case Display::negativeButtonBackgroundFillIndex:
         case Display::defaultButtonBackgroundFillIndex:
+          if (HasTheming()) {
+            RECT content,extent,clip;
+
+            if (draw->disabled) {
+              style=PBS_DISABLED;
+            }
+            else if (draw->selected) {
+              style=PBS_PRESSED;
+            }
+            else if (draw->activated) {
+              style=PBS_HOT;
+            }
+            else if (draw->focused) {
+              style=PBS_DEFAULTED;
+            }
+            else {
+              style=PBS_NORMAL;
+            }
+
+            content.left=xOff-3;
+            content.right=xOff+width+6;
+            content.top=yOff-3;
+            content.bottom=yOff+height+6;
+
+            MyGetThemeBackgroundExtent(buttonTheme,dc,BP_PUSHBUTTON,style,&content,&extent);
+
+            clip.left=x;
+            clip.right=x+w;
+            clip.top=y;
+            clip.bottom=y+h;
+
+            MyDrawThemeBackground(buttonTheme,dc,BP_PUSHBUTTON,style,&extent,&clip);
+
+            return;
+          }
+          brush=::GetSysColorBrush(COLOR_BTNFACE);
+          break;        
         case Display::scrollButtonBackgroundFillIndex:
           if (HasTheming()) {
             RECT content,extent,clip;
@@ -1509,6 +1517,13 @@ namespace Lum {
             rightBorder=extent.right;
             topBorder=-extent.top;
             bottomBorder=extent.bottom;
+
+            // 1 pixel for the focus frame and two pixel for the space between frame and content
+            leftBorder+=1+2;
+            rightBorder+=1+2;
+            topBorder+=1+2;
+            bottomBorder+=1+2;
+
             alpha=true;
           }
           else {
@@ -1877,6 +1892,13 @@ namespace Lum {
             bottomBorder=2;
           }
           break;
+        case Display::labelFrameIndex:
+          leftBorder=1;
+          rightBorder=leftBorder;
+          topBorder=1;
+          bottomBorder=topBorder;
+          alpha=true;
+          break;
         }
 
         minWidth=leftBorder+rightBorder;
@@ -1925,6 +1947,22 @@ namespace Lum {
             draw->PushClipEnd();
             /* ignore */ MyDrawThemeBackground(buttonTheme,dc,BP_PUSHBUTTON,style,&rect,NULL);
             draw->PopClip();
+
+            if (draw->focused) {
+              x+=leftBorder-1-2;
+              y+=topBorder-1-2;
+              w-=(leftBorder-1-2)+(rightBorder-1-2);
+              h-=(topBorder-1-2)+(bottomBorder-1-2);
+
+              draw->PushForeground(GetSysColor(COLOR_BTNTEXT));
+              draw->PushDash("\001\001",2,::Lum::OS::DrawInfo::fDash);
+              draw->DrawLine(x          ,y,      x+w-1,y);
+              draw->DrawLine(x+w-1,y,            x+w-1,y+h-1);
+              draw->DrawLine(x+w-1,y+h-1,x,            y+h-1);
+              draw->DrawLine(x          ,y+h-1,x,      y);
+              draw->PopDash();
+              draw->PopForeground();
+            }
           }
           else {
             draw->PushClipBegin(x,y,w,h);
@@ -2387,6 +2425,19 @@ namespace Lum {
             }
           }
           break;
+        case Display::labelFrameIndex:
+            if (draw->focused) {
+              ///* ignore */ DrawFocusRect(dc,&rect);
+              draw->PushForeground(GetSysColor(COLOR_BTNTEXT));
+              draw->PushDash("\001\001",2,::Lum::OS::DrawInfo::fDash);
+              draw->DrawLine(x          ,y,      x+w-1,y);
+              draw->DrawLine(x+w-1,y,            x+w-1,y+h-1);
+              draw->DrawLine(x+w-1,y+h-1,x,            y+h-1);
+              draw->DrawLine(x          ,y+h-1,x,      y);
+              draw->PopDash();
+              draw->PopForeground();
+            }
+          break;
         default:
           assert(false);
           break;
@@ -2482,9 +2533,8 @@ namespace Lum {
         frame[Display::groupFrameIndex]=new Frame(Display::groupFrameIndex);
         frame[Display::menuStripFrameIndex]=emptyFrame;
         frame[Display::menuPulldownFrameIndex]=new Frame(Display::menuPulldownFrameIndex);
+        frame[Display::labelFrameIndex]=new Frame(Display::labelFrameIndex);
 
-        image[Display::checkImageIndex]=new ThemeImage(Display::checkImageIndex);
-        image[Display::radioImageIndex]=new ThemeImage(Display::radioImageIndex);
         image[Display::hknobImageIndex]=new ThemeImage(Display::hknobImageIndex);
         image[Display::vknobImageIndex]=new ThemeImage(Display::vknobImageIndex);
         image[Display::arrowLeftImageIndex]=new ThemeImage(Display::arrowLeftImageIndex);
@@ -2507,8 +2557,9 @@ namespace Lum {
         image[Display::bottomSliderImageIndex]=new ThemeImage(Display::bottomSliderImageIndex);
         image[Display::menuDividerImageIndex]=new ThemeImage(Display::menuDividerImageIndex);
         image[Display::menuSubImageIndex]=new ThemeImage(Display::menuSubImageIndex);
+        image[Display::menuCheckImageIndex]=new ThemeImage(Display::menuCheckImageIndex);
+        image[Display::menuRadioImageIndex]=new ThemeImage(Display::menuRadioImageIndex);
         image[Display::closeImageIndex]=new ThemeImage(Display::closeImageIndex);
-        image[Display::ledImageIndex]=new ThemeImage(Display::ledImageIndex);
       }
 
       Win32Theme::~Win32Theme()
@@ -2564,14 +2615,199 @@ namespace Lum {
         return ((metric.tmAscent+metric.tmDescent)*72)/GetDeviceCaps(dynamic_cast<Display*>(OS::display)->hdc,LOGPIXELSY);
       }
 
-      OS::Theme::ScrollKnobMode Win32Theme::GetScrollKnobMode() const
+      void Win32Theme::GetObjectSize(ObjectType object,
+                                     size_t& width,
+                                     size_t& height) const
       {
-        return OS::Theme::scrollKnobModeSingle;
+        switch (object) {
+        case objCheck:
+          width=GetSystemMetrics(SM_CXMENUCHECK);
+          height=GetSystemMetrics(SM_CYMENUCHECK);
+          break;
+        case objRadio:
+          width=GetSystemMetrics(SM_CXMENUCHECK);
+          height=GetSystemMetrics(SM_CYMENUCHECK);
+          break;
+        case objLED:
+          width=GetSystemMetrics(SM_CXMENUCHECK);
+          height=GetSystemMetrics(SM_CYMENUCHECK);
+          break;
+        }
       }
 
-      OS::Theme::ToolbarPosition Win32Theme::GetToolbarPosition() const
+      bool Win32Theme::NeedsObjectParentDraw(ObjectType object) const
       {
-        return OS::Theme::toolbarTop;
+        switch (object) {
+        case objCheck:
+        case objRadio:
+        case objLED:
+          return true;
+        }
+        
+        assert(false);
+      }
+
+      void Win32Theme::DrawObject(OS::DrawInfo* draw,
+                                  ObjectType object,
+                                  int x, int y, size_t width, size_t height,
+                                  int cX, int cY, size_t cWidth, size_t cHeight)
+      {
+        HDC  dc=dynamic_cast<DrawInfo*>(draw)->dc;
+        RECT rect;
+        int  style;
+
+        rect.left=x;
+        rect.top=y;
+        rect.right=x+width;
+        rect.bottom=y+height;
+
+        SetTextAlign(dc,TA_TOP+TA_LEFT);
+
+        switch (object) {
+        case objCheck:
+          if (HasTheming()) {
+            if (draw->selected) {
+              if (draw->activated) {
+                style=CBS_CHECKEDHOT;
+              }
+              else if (draw->disabled) {
+                style=CBS_CHECKEDDISABLED;
+              }
+              else {
+                style=CBS_CHECKEDNORMAL;
+              }
+            }
+            else if (draw->activated) {
+              style=CBS_UNCHECKEDHOT;
+            }
+              else if (draw->disabled) {
+                style=CBS_UNCHECKEDDISABLED;
+              }
+            else {
+              style=CBS_UNCHECKEDNORMAL;
+            }
+
+            /* ignore */ MyDrawThemeBackground(buttonTheme,dc,BP_CHECKBOX,style,&rect,NULL);
+          }
+          else {
+           /* ignore */ ::DrawFrameControl(dc,&rect,
+                                            DFC_BUTTON,
+                                            DFCS_BUTTONCHECK | (draw->selected ? DFCS_CHECKED : 0));
+          }
+          break;
+        case objRadio:
+          if (HasTheming()) {
+            if (draw->selected) {
+              if (draw->activated) {
+                style=RBS_CHECKEDHOT;
+              }
+              else if (draw->disabled) {
+                style=RBS_CHECKEDDISABLED;
+              }
+              else {
+                style=RBS_CHECKEDNORMAL;
+              }
+            }
+            else if (draw->activated) {
+              style=RBS_UNCHECKEDHOT;
+            }
+            else if (draw->disabled) {
+              style=RBS_UNCHECKEDDISABLED;
+            }
+            else {
+              style=RBS_UNCHECKEDNORMAL;
+            }
+
+            /* ignore */ MyDrawThemeBackground(buttonTheme,dc,BP_RADIOBUTTON,style,&rect,NULL);
+          }
+          else {
+            /* ignore */ ::DrawFrameControl(dc,&rect,
+                                            DFC_BUTTON,
+                                            DFCS_BUTTONRADIO | (draw->selected ? DFCS_CHECKED : 0));
+          }
+          break;
+        case objLED:
+          if (draw->selected) {
+            draw->PushForeground(GetSysColor(COLOR_HIGHLIGHT));
+            draw->FillArc(x,y,width-1,height-1,0*64,360*64);
+            draw->PopForeground();
+          }
+          else {
+            draw->PushForeground(GetSysColor(COLOR_3DSHADOW));
+            draw->FillArc(x,y,width-1,height-1,0*64,360*64);
+            draw->PopForeground();
+          }
+
+          draw->PushForeground(GetSysColor(COLOR_3DHIGHLIGHT));
+          draw->DrawArc(x+width / 4,y+height / 4,
+                       width / 2,height / 2,60*64,90*64);
+          draw->PopForeground();
+
+          draw->PushForeground(GetSysColor(COLOR_3DSHADOW));
+          draw->DrawArc(x,y,width-1,height-1,0*64,360*64);
+          draw->PopForeground();
+          break;
+        }
+
+        SetTextAlign(dc,TA_BASELINE+TA_LEFT);
+      }
+
+      size_t Win32Theme::GetSpaceHorizontal(Display::Space space) const
+      {
+        switch (space) {
+        case Display::spaceWindowBorder:
+          return 12;
+        case Display::spaceInterGroup:
+          return 18;
+        case Display::spaceGroupIndent:
+          return 12;
+        case Display::spaceInterObject:
+          return 6;
+        case Display::spaceObjectBorder:
+          return 6;
+        case Display::spaceIntraObject:
+          return 6;
+        case Display::spaceLabelObject:
+          return 12;
+        case Display::spaceObjectDetail:
+          return 16;
+        }
+
+        return 1;
+      }
+
+      size_t Win32Theme::GetSpaceVertical(Display::Space space) const
+      {
+        switch (space) {
+        case Display::spaceWindowBorder:
+          return 12;
+        case Display::spaceInterGroup:
+          return 18;
+        case Display::spaceGroupIndent:
+          return 12;
+        case Display::spaceInterObject:
+          return 6;
+        case Display::spaceObjectBorder:
+          return 3;
+        case Display::spaceIntraObject:
+          return 6;
+        case Display::spaceLabelObject:
+          return 6;
+        case Display::spaceObjectDetail:
+          return 16;
+        }
+
+        return 1;
+      }
+
+      size_t Win32Theme::GetDragStartSensitivity() const
+      {
+        return 6;
+      }
+
+      size_t Win32Theme::GetMouseClickHoldSensitivity() const
+      {
+        return 5;
       }
 
       size_t Win32Theme::GetMinimalButtonWidth() const
@@ -2599,6 +2835,21 @@ namespace Lum {
       {
         return 2;
       }
+
+      OS::Theme::ScrollKnobMode Win32Theme::GetScrollKnobMode() const
+      {
+        return OS::Theme::scrollKnobModeSingle;
+      }
+
+      OS::Theme::ToolbarPosition Win32Theme::GetToolbarPosition() const
+      {
+        return OS::Theme::toolbarTop;
+      }
+
+      bool Win32Theme::ShowButtonImages() const
+      {
+        return false;
+      }
     }
   }
 }
