@@ -1225,7 +1225,7 @@ namespace Lum {
 
         EvaluateDisplaySize();
 
-        dpi=XDisplayWidth(display,scrNum)/(XDisplayWidthMM(display,scrNum)/25.4);
+        dpi=XDisplayHeight(display,scrNum)/(XDisplayHeightMM(display,scrNum)/25.4);
         /*
         int            eventBase,errorBase;
 
@@ -1324,8 +1324,7 @@ namespace Lum {
 
 #if defined(HAVE_LIB_GTK)
         if ((driver->GetDriverName()==L"X11" ||
-             driver->GetDriverName()==L"CairoX11") &&
-             prefs->theme==L"Gtk") {
+             driver->GetDriverName()==L"CairoX11")) {
           theme=new GtkTheme(this);
         }
         else {
