@@ -25,6 +25,9 @@ namespace Lum {
   : boolModel(NULL),intModel(NULL),index(1),active(active)
   {
     SetCanFocus(active);
+    if (active) {
+      RequestFocus();
+    }
     SetRequestLabelFocus(true);
   }
 
@@ -174,27 +177,18 @@ namespace Lum {
     Control::Resync(model,msg);
   }
 
-  ImageState::Prefs::Prefs(const std::wstring& name)
-  : State::Prefs(name)
-  {
-    // no code
-  }
-
-  ImageState::ImageState(bool active)
+  ImageState::ImageState(OS::Theme::ObjectType objectType, bool active)
   : State(active),
-    image(NULL)
+    object(objectType)
   {
     // no code
   }
 
   void ImageState::CalcSize()
   {
-    image=dynamic_cast<Prefs*>(prefs)->image;
+    SetCanDrawMouseActive(true/*image->GetDrawCap() & OS::Image::drawsActivated*/);
 
-    SetCanDrawMouseActive(image->GetDrawCap() & OS::Image::drawsActivated);
-
-    minWidth=image->GetWidth();
-    minHeight=image->GetHeight();
+    OS::display->GetTheme()->GetObjectSize(object,minWidth,minHeight);
     width=minWidth;
     height=minHeight;
 
@@ -203,11 +197,13 @@ namespace Lum {
 
   void ImageState::DrawContent(int x, int y, size_t w, size_t h)
   {
-    if (image->GetAlpha()) {
-      DrawBackground(x,y,width,height);
+    if (OS::display->GetTheme()->NeedsObjectParentDraw(object)) {
+      DrawParentBackground(x,y,width,height);
     }
 
-    image->Draw(GetDrawInfo(),this->x,this->y,this->width,this->height);
+    OS::display->GetTheme()->DrawObject(GetDrawInfo(),object,
+                                        this->x,this->y,this->width,this->height,
+                                        x,y,w,h);
   }
 
 }
