@@ -192,12 +192,6 @@ namespace Lum {
     hScroller->SetModel(table->GetHAdjustment());
     hScroller->CalcSize();
 
-    if (RequestedFocus()) {
-      /* Delegate focusing to the real table */
-      table->RequestFocus();
-      UnrequestFocus();
-    }
-
     object->CalcSize();
 
     // Initial dimension equal to table dimension
